# br_jaspersoft_app

[![Build Status](https://git.devops.broadridge.net/dba-tss-ind/devops/chef/cookbooks/br_jaspersoft_app/badges/master/pipeline.svg)](https://git.devops.broadridge.net/dba-tss-ind/devops/chef/cookbooks/br_jaspersoft_app/commits/master)

## Overview
This cookbook facilitates the process of installing Postgres Database 10.4 (if Required) and JaspersoftServer 6.2 along with Apache tomcat server and Java 1.7 on Linux platforms.
Note: This cookbook installs Postgres Binaries and creates Jasperserver metadata related databases.

## Platforms
RHEL 7+ (and its derivatives)

## Prerequisites and Dependencies
Minimum server level requirements
  - Linux server with RHEL 7 (or higher)
  - RAM: 4GB (recommend 8GB or more)
  - Storage: A separate device and volume group is required for the JasperServer Home with a minimum recommended size of 20G.

## Cookbook Dependencies
  - `br_yum` cookbook is required for installing OS packages.
  - `br_lvm` cookbook is required to lvm creation.
  - `br_aws` cookbook is required for paramterstore passwords
  - `br_apm_tomcat` cookbook is required to enable APM monitoring.

## Usage
**To use this cookbook, Place a dependency in your `metadata.rb`**

`depends 'br_jaspersoft_app'`

**Ensure all attributes (described below with example values) are defined in your local attributes `default.rb` file.**

**Add an include line in your recipe to call this cookbook.**

`include_recipe br_jaspersoft_app::default`

## Required Attributes
`jasper_mount` -> **The root location of the Jaspersoft installation.**
  - **Below are descriptions and example values for the required parameters.**
    - `default['br_jaspersoft_app']['jasper_mount'] = '/apps'`
    - `default['br_jaspersoft_app']['postgres_server_required'] = 'Yes'`
    - `default['br_jaspersoft_app']['postgres_binary_version'] = '10.4-1'`
    - `default['br_jaspersoft_app']['postgres_os_family'] = 'linux'`
    - `default['br_jaspersoft_app']['postgres_os_bit'] = 'x64'`
    - `default['br_jaspersoft_app']['postgres_user'] = 'postgres'`
    - `default['br_jaspersoft_app']['postgres_binary_version'] = '10.4'`
    - `default['br_jaspersoft_app']['postgres_host'] = 'localhost'`
    - `default['br_jaspersoft_app']['postgres_port'] = '9400'`
    - `default['br_jaspersoft_app']['postgres_dbuser'] = 'postgres'`
    - `default['br_jaspersoft_app']['postgres_locale'] = 'en_US.utf8'`
    - `default['br_jaspersoft_app']['jasper_binary_version'] = '6.2.0'`
    - `default['br_jaspersoft_app']['apache_binary_version'] = '7.0.72'`
    - `default['br_jaspersoft_app']['java_binary_version'] = '7u75'`
    - `default['br_jaspersoft_app']['java_version'] = '1.7.0_75'`
    - `default['br_jaspersoft_app']['jasper_os_bit'] = 'x64'`
    - `default['br_jaspersoft_app']['jasper_os_family'] = 'linux'`
    - `default['br_jaspersoft_app']['jasper_user'] = 'jspadmin'`
    - `default['br_jaspersoft_app']['apache_version'] = 'tomcat7'`
    - `default['br_jaspersoft_app']['apache_server_port'] = '8072'`
    - `default['br_jaspersoft_app']['apache_http_port'] = '8082'`
    - `default['br_jaspersoft_app']['apache_ajp_port'] = '8092'`
    - `default['br_jaspersoft_app']['apache_https_port'] = '8443'`
    - `default['br_jaspersoft_app']['apache_ajpre_port'] = '8443'`
    - `default['br_jaspersoft_app']['jasper_dbtype'] = 'postgresql'`
    - `default['br_jaspersoft_app']['jasper_dbhost'] = 'localhost'`
    - `default['br_jaspersoft_app']['jasper_dbuser'] = 'jaspersql'`
    - `default['br_jaspersoft_app']['jasper_schema'] = 'jasper_schema'`
    - `default['br_jaspersoft_app']['jasper_dbport'] = '9400'`
    - `default['br_jaspersoft_app']['jasper_dbdbname'] = 'pgsgptm02'`
    - `default['br_jaspersoft_app']['jasper_instance'] = '02'`
    - `default['br_jaspersoft_app']['jasperrmiport'] = '10992'`
    - `default['br_jaspersoft_app']['jasperrmihost'] = 'localhost'`
    - `default['br_jaspersoft_app']['jasperrmijmxname'] = 'jasperserver'`
    - `default['br_jaspersoft_app']['jasper_webname'] = 'gptm-reports'`
    - `default['br_jaspersoft_app']['jasper_client_name'] = 'gptm'`

## Other parameters
  - **Source URL:** Source location of software run file(i.e. Nexus).
    - `default['br_jaspersoft_app']['source_Main_url'] = 'https://nexus.devops.broadridge.net/repository/3rdParty_RAW/Jasper_soft'`
  - **Staging Directory:** Staging location for unzipping software files.
    - `default['br_jaspersoft_app']['temp_download_loc'] = '/apps/downloads'`