jasper = node['jaspersoft_app']['jasper']
jasper_mnt = jasper['jasper_mount']
jasper_usr = jasper['jasper_user']
jasper_grp = jasper['jasper_group_main']
# tomcat_home = "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}"
# jasper_dbcname = "#{jasper['app_db']}.#{node['ec2']['placement_availability_zone'].chop}.#{jasper['db_hostdns']}"
# jasper_dbcname = jasper['jasper_dbhost'] if jasper['jasper_server_type'] == 'new'

# #######required tools##############
jsp_packages = %w(
  unzip
  dos2unix
  telnet
 )
jsp_packages.each do |p|
  package p
end

# #################### Aws Parameter Store configuration################################
aws_ssm_parameter_store jasper['ssm_paramstore_path'] do
  with_decryption true
  return_key 'jasper_paramstore'
  sensitive true
  action :nothing
end.run_action(:get)

# #################### Aws Parameter Store for nexus################################
aws_ssm_parameter_store jasper['nexus_ssm_paramstore_path'] do
  with_decryption true
  return_key 'jasper_nexus_paramstore'
  sensitive true
  action :nothing
end.run_action(:get)

# ############Users and Groups Creation For Apache, Jasper and Postgres#############
user jasper_usr do
  comment 'A Jaspersoft user'
  shell '/bin/bash'
  manage_home true
  system true
  sensitive true
end

# ==================================================================================
user jasper['postgres_user'] do
  comment 'A Postgres user'
  shell '/bin/bash'
  manage_home true
  system true
  sensitive true
end

# ############Folder Creation For Apache, Jasper and Postgres#############
[jasper_mnt, jasper['postgres_app_dir'], jasper['postgres_data_dir'], jasper['postgres_install_dir'], jasper['apache_dir'], jasper['temp_download_loc'], "#{jasper['temp_download_loc']}/apache", "#{jasper['temp_download_loc']}/jasper", "#{jasper['temp_download_loc']}/postgres"].each do |dir|
  directory dir do
    action :create
  end
end

# ############Folder permissions For Apache, Jasper and Postgres#############
[jasper_mnt, jasper['apache_dir'], jasper['temp_download_loc'], "#{jasper['temp_download_loc']}/apache", "#{jasper['temp_download_loc']}/jasper", "#{jasper['temp_download_loc']}/postgres", "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}"].each do |dir|
  directory dir do
    user jasper_usr
    group jasper_grp
    recursive true
    mode '775'
  end
end

[jasper['postgres_app_dir'], jasper['postgres_data_dir'], jasper['postgres_install_dir']].each do |dir|
  directory dir do
    owner jasper['postgres_user']
    group jasper['postgres_user']
    recursive true
    mode '775'
  end
end

# ############Binaries Download and Extractions For Apache, Jasper and Postgres#############
["apache-tomcat-#{jasper['apache_binary_version']}.tar.gz", "jdk-#{jasper['java_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.tar.gz", "jasperreports-server-#{jasper['jasper_binary_version']}-bin.zip", "postgresql-#{jasper['postgres_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.run"].each do |download_file|
  remote_file "#{jasper['temp_download_loc']}/#{download_file}" do
    source "#{node.run_state['jasper_nexus_paramstore']}/#{download_file}"
    action :create_if_missing
    notifies :run, "execute[extract-#{download_file}]", :immediately
    mode '755'
  end
end

execute "extract-apache-tomcat-#{jasper['apache_binary_version']}.tar.gz" do
  cwd "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}"
  command <<-CMD
  tar -zxvf "#{jasper['temp_download_loc']}/apache-tomcat-#{jasper['apache_binary_version']}.tar.gz"
  CMD
  user jasper_usr
  group jasper_grp
  action :nothing
end

execute "extract-jdk-#{jasper['java_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.tar.gz" do
  cwd jasper['apache_dir']
  command <<-CMD
  tar -zxvf "#{jasper['temp_download_loc']}/jdk-#{jasper['java_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.tar.gz"
  CMD
  user jasper_usr
  group jasper_grp
  action :nothing
end

execute "extract-jasperreports-server-#{jasper['jasper_binary_version']}-bin.zip" do
  cwd jasper['apache_dir']
  command <<-CMD
  unzip -q -o #{jasper['temp_download_loc']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin.zip -d #{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}
  chmod -R 755 #{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}
  CMD
  user jasper_usr
  group jasper_grp
  action :nothing
end

execute "extract-postgresql-#{jasper['postgres_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.run" do
  cwd jasper['temp_download_loc']
  command <<-CMD
  chmod 775 postgresql-#{jasper['postgres_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.run
  CMD
  action :nothing
end

# ############Folder permissions For Apache, Jasper and Postgres#############
[jasper_mnt, jasper['apache_dir'], jasper['temp_download_loc'], "#{jasper['temp_download_loc']}/apache", "#{jasper['temp_download_loc']}/jasper", "#{jasper['temp_download_loc']}/postgres", "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}"].each do |dir|
  directory dir do
    user jasper_usr
    group jasper_grp
    recursive true
    mode '775'
  end
end

# ############Template Files copy to server For Apache, Jasper and Postgres#############
template "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin/buildomatic/default_master.properties" do
  source 'default_master.properties.erb'
  owner jasper_usr
  group jasper_grp
  sensitive true
  variables apache_version: jasper['apache_version'],
            apache_dir: "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}",
            jasper_db_type: jasper['jasper_dbtype'],
            jasper_db_host: 'localhost',
            jasper_db_user: jasper['jasper_dbuser'],
            jasper_db_password: node.run_state['jasper_paramstore'],
            jasper_db_port: jasper['jasper_dbport'],
            jasper_db_dbname: jasper['jasper_dbdbname'],
            jasper_web_name: jasper['jasper_webname'],
            jasperrmiport: jasper['jasperrmiport'],
            jasperrmijmxname: jasper['jasperrmijmxname'],
            jasperrmihost: jasper['jasperrmihost']
  action :create
end

# apache tomcat users file copy for apache-tomcat
template "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}/conf/tomcat-users.xml" do
  source 'tomcat-users.xml.erb'
  owner jasper_usr
  group jasper_grp
  sensitive true
  mode '0750'
  variables apache_user: jasper['apache_user'],
            apache_password: node.run_state['jasper_paramstore']
  action :create
end

# apache server config file copy for apache-tomcat
if jasper['apache_https_enable'] == 'yes'
  template "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}/conf/server.xml" do
    source 'server.xml.erb'
    owner jasper_usr
    group jasper_grp
    sensitive true
    mode '0750'
    variables apache_server_port: jasper['apache_server_port'],
              apache_http_port: jasper['apache_http_port'],
              apache_https_port: jasper['apache_https_port'],
              apache_ajp_port: jasper['apache_ajp_port'],
              apache_ajpre_port: jasper['apache_ajpre_port']
    action :create
  end
else
  template "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}/conf/server.xml" do
    source 'server.xml_no_https.erb'
    owner jasper_usr
    group jasper_grp
    sensitive true
    mode '0750'
    variables apache_server_port: jasper['apache_server_port'],
              apache_http_port: jasper['apache_http_port'],
              apache_https_port: jasper['apache_https_port'],
              apache_ajp_port: jasper['apache_ajp_port'],
              apache_ajpre_port: jasper['apache_ajpre_port']
    action :create
  end
end

# apache server installation as service
template "/etc/systemd/system/tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}.service" do
  source 'tomcat.service.erb'
  variables java_home: "#{jasper['apache_dir']}/jdk#{jasper['java_version']}",
            catalina_home: "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}",
            pid_details: "tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}",
            service_user: jasper_usr,
            service_group: jasper_grp,
            apm_name: jasper['apm_agent_name']
  notifies :run, 'execute[Apache Process initiate]', :delayed
  action :create
end

remote_file "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/apache-tomcat-#{jasper['apache_binary_version']}/conf/jspadmin.jks" do
  source "#{node.run_state['jasper_nexus_paramstore']}/jspadmin.jks"
  action :create_if_missing
  mode '755'
end

# apache server initialization as service
execute 'Apache Process initiate' do
  command <<-CMD
    sudo systemctl daemon-reload
    sudo systemctl enable tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}
  CMD
  notifies :start, "service[tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}]", :immediately
  action :nothing
end

# jasper user bash profile file
template "#{jasper['jasper_home_dir']}/.bash_profile" do
  source 'jspadmin_bash.erb'
  sensitive true
  owner jasper_usr
  group jasper_grp
  mode '0750'
  variables apache_dir: jasper['apache_dir'],
            jasper_client_instance: "#{jasper['jasper_client_name']}#{jasper['jasper_instance']}",
            java_version: jasper['java_version']
  action :create
end

# ##############Get pem file from Nexus###############
# remote_file "#{jasper['apache_dir']}/jdk#{jasper['java_version']}/jre/lib/security/rds-ca-2019-root.pem" do
#   source "#{node.run_state['jasper_nexus_paramstore']}/rds-ca-2019-root.pem"
#   action :create_if_missing
#   mode '755'
#   owner jasper_usr
#   group jasper_grp
#   mode '0744'
#   show_progress true
#   notifies :run, 'execute[Rds_Cert_Installation]', :immediately
# end

# # ##############Import certs to truststor###############
# execute 'Rds_Cert_Installation' do
#   cwd "#{jasper['apache_dir']}/jdk#{jasper['java_version']}/jre/lib/security"
#   command <<-CMD
#   openssl x509 -in rds-ca-2019-root.pem -out rds-ca-2019-root.der -outform der
#   #{jasper['apache_dir']}/jdk#{jasper['java_version']}/jre/bin/keytool -keystore cacerts -alias rds-postgresql -import -file rds-ca-2019-root.der -storepass changeit -trustcacerts -noprompt
#   CMD
#   user jasper_usr
#   group jasper_grp
#   action :nothing
#   live_stream true
# end

# #################context file creation################################
# template "#{jasper['apache_dir']}/context.xml" do
#   source 'context.erb'
#   owner jasper_usr
#   group jasper_grp
#   sensitive true
#   mode '0750'
#   variables metadata_db_driverClassName: jasper['metadata_db_driverClassName'],
#             jasperrmijmxname: jasper['jasperrmijmxname'],
#             jasper_db_password: node.run_state['jasper_paramstore'],
#             jasper_dbhost: jasper['jasper_dbhost'],
#             jasper_dbport: jasper['jasper_dbport'],
#             jasper_dbdbname: jasper['jasper_dbdbname'],
#             jasper_dbuser: jasper['jasper_dbuser']
# end

# postgres user bash profile file
template "#{jasper['postgres_home_dir']}/.bash_profile" do
  source 'postgres_bash.erb'
  sensitive true
  owner jasper['postgres_user']
  group jasper['postgres_user']
  mode '0750'
  variables postgres_data_dir: jasper['postgres_data_dir'],
            postgres_app_dir: jasper['postgres_install_dir'],
            postgres_temp_dir: jasper['postgres_temp_dir']
  notifies :run, 'execute[Postgres DB Installation]', :delayed
  only_if { jasper['postgres_server_required'] == 'yes' }
  action :create
end

# postgres masterDB pgpass file
template "#{jasper['postgres_home_dir']}/.pgpass" do
  source 'pgpass.erb'
  sensitive true
  owner jasper['postgres_user']
  group jasper['postgres_user']
  mode '0600'
  variables postgres_host: jasper['postgres_host'],
            postgres_port: jasper['postgres_port'],
            postgres_db: jasper['postgres_dbuser'],
            postgres_dbuser: jasper['postgres_dbuser'],
            postgres_dbpwd: node.run_state['jasper_paramstore'],
            jasper_dbdbname: jasper['jasper_dbdbname'],
            jasper_dbuser: jasper['jasper_dbuser'],
            jasper_dbpwd: node.run_state['jasper_paramstore']
  notifies :run, 'execute[Postgres DB Installation]', :delayed
  only_if { jasper['postgres_server_required'] == 'yes' }
  action :create
end

# postgres config Option file
template "#{jasper['temp_download_loc']}/postgres_config" do
  source 'postgres_config.erb'
  sensitive true
  owner jasper['postgres_user']
  group jasper['postgres_user']
  mode '0755'
  variables postgres_install_mode: jasper['postgres_install_mode'],
            postgres_install_dir: jasper['postgres_install_dir'],
            postgres_data_dir: jasper['postgres_data_dir'],
            postgres_server_port: jasper['jasper_dbport'],
            postgres_user: jasper['postgres_user'],
            postgres_locale: jasper['postgres_locale']
  notifies :run, 'execute[Postgres DB Installation]', :delayed
  only_if { jasper['postgres_server_required'] == 'yes' }
  action :create
end

# ############PostgreSQL server installation#############
execute 'Postgres DB Installation' do
  cwd jasper['temp_download_loc']
  command <<-CMD
  chmod 775 "#{jasper['temp_download_loc']}/postgresql-#{jasper['postgres_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.run"
  export TEMP="#{jasper['postgres_temp_dir']}"
  "#{jasper['temp_download_loc']}/postgresql-#{jasper['postgres_binary_version']}-#{jasper['postgres_os_family']}-#{jasper['postgres_os_bit']}.run" --optionfile "#{jasper['temp_download_loc']}/postgres_config"
  mv "#{jasper['postgres_data_dir']}/pg_hba.conf" "#{jasper['postgres_data_dir']}/pg_hba_bkp.conf"
  CMD
  notifies :create, "cookbook_file[#{jasper['postgres_data_dir']}/pg_hba.conf]", :delayed
  action :nothing
end

cookbook_file "#{jasper['postgres_data_dir']}/pg_hba.conf" do
  mode '0777'
  source 'pg_hba.conf'
  notifies :run, 'execute[Postgres post setup]', :delayed
  action :nothing
end

# ############PostgreSQL server post setup (Create DB, Create USER & Create ROLES#############
execute 'Postgres post setup' do
  cwd jasper['postgres_home_dir']
  user jasper['postgres_user']
  command <<-CMD
    #{jasper['postgres_install_dir']}/bin/pg_ctl reload -D "#{jasper['postgres_data_dir']}"
    #{jasper['postgres_install_dir']}/bin/psql -p"#{jasper['postgres_port']}" -w -c "create user "#{jasper['jasper_dbuser']}" with password '"#{node.run_state['jasper_paramstore']}"';"
    #{jasper['postgres_install_dir']}/bin/psql -p"#{jasper['postgres_port']}" -w -c "alter user "#{jasper['jasper_dbuser']}" createdb;"
    #{jasper['postgres_install_dir']}/bin/psql -d "#{jasper['jasper_dbdbname']}" -U"#{jasper['jasper_dbuser']}" -p"#{jasper['postgres_port']}" -w -c "create schema "#{jasper['jasper_schema']}" authorization "#{jasper['jasper_dbuser']}";"
    #{jasper['postgres_install_dir']}/bin/psql -d "#{jasper['jasper_dbdbname']}" -U"#{jasper['jasper_dbuser']}" -p"#{jasper['postgres_port']}" -w -c "alter role "#{jasper['jasper_dbuser']}" set search_path="#{jasper['jasper_schema']}";"
    #{jasper['postgres_install_dir']}/bin/psql -p"#{jasper['postgres_port']}" -w -c "DROP DATABASE IF EXISTS #{jasper['jasper_dbdbname']};"
  CMD
  notifies :run, 'execute[Jasper-Soft Installation]', :delayed
  action :nothing
end

# ############JASPERSOFT server installation#############
execute 'Jasper-Soft Installation' do
  environment JAVA_HOME: "#{jasper['apache_dir']}/jdk#{jasper['java_version']}"
  cwd "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin/buildomatic"
  user jasper['jasper_user']
  if jasper['jasper_server_type'] == 'new'
    command <<-CMD
      #{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin/buildomatic/js-install.sh #{jasper['jasper_install_type']}
      cp "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin/jasperserver.license" "#{jasper['jasper_home_dir']}/"
    CMD
  end
  if jasper['jasper_server_type'] == 'join'
    command <<-CMD
      #{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin/buildomatic/js-ant deploy-webapp-pro
      cp "#{jasper['apache_dir']}/jasper-server-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}/jasperreports-server-#{jasper['jasper_binary_version']}-bin/jasperserver.license" "#{jasper['jasper_home_dir']}/"
    CMD
  end
  if jasper['postgres_server_required'] == 'yes'
    notifies :restart, "service[tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}]", :immediately
    action :nothing
    notifies :sleep, 'chef_sleep[wait for the apache service to start]'
  end
  if jasper['postgres_server_required'] == 'no'
    notifies :restart, "service[tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}]", :immediately
    action :run
    notifies :sleep, 'chef_sleep[wait for the apache service to start]'
  end
end

# apache services resource
service "tomcat-#{jasper['jasper_client_name']}#{jasper['jasper_instance']}" do
  action :nothing
end

chef_sleep 'wait for the apache service to start' do
  seconds 30
  action :nothing
end
